/**
		@file 	config.cpp
		@date	2009/08/25
		@author	최영업(cyu0070@naver.com) Terapi
		@brief  Server접속시에 필요한 Config파일을 읽어드리기위한 함수군

*/
using namespace std;
#include "config.h"

int  Conf::sServerPort             =0; ///<서버포트 
int  Conf::sMaxUsers               =0; ///<총접속 가능한 유저수
int  Conf::sLoginTimeOut           =0; ///<로긴 타임아웃을 위한 값
int  Conf::sIdleTimeOut            =0; ///<접속후 아이들타임을위한 값
int  Conf::sDaemonMode             =1; ///<데몬모드
int  Conf::sRealChangeRoot         =1;
int  Conf::sLoginInfoCount         =0; ///<접속자수
int  Conf::sBanCount               =0; ///<금지접속자수
unsigned int Conf::sTransferTimeOut=240;

char Conf::sIniFileName[512];
char Conf::sMotdFile[MAXPATHLEN+4];
char Conf::sByeMsgFile[MAXPATHLEN+4];
char Conf::s_ServerRoot[MAXPATHLEN+4];
char Conf::sBanMessage[72];
FILE Conf::*logfile = NULL;

char const *options[OPTS_COUNT]=
{
	"SERVERROOT",   "MAXUSERS", "PORT", "LOGIN-TIMEOUT", "USER", "TIMEOUT", "MODE", "LOG","MOTD","BYEMSG", "BAN", "BANMSG", "CHANGEROOTTYPE"
};

/**
		 @return		options[]배열안에 들어있는 문자열과 비교하여서 맞으면 해당하는 배열값
*/
int Conf::Optindex(char *opt)
{
	int n=0;
	while (opt[n]==' ' && n<strlen(opt)) 
		n++;

	if (opt[n]=='#' || opt[n]==0) 
		return OPT_COMMENT;


	for (n=0;n<OPTS_COUNT;n++)
		if (!strcasecmp(opt,options[n])) 
			return n;

	return -1;
}
/***
		 @description	conf파일을 읽어드리는 함수
		 @return		참이면 0 define된 값 max군으로 지정된값보다 크면 -1 return

  **/
int Conf::Readini()
{
	Conf cn;
	FILE *fp;
	char line[2048],opt[OPT_MAXLEN+4],arg[ARG_MAXLEN+4];
    
	strcpy(sIniFileName,DEFAULT_CONFPATH);

	if ( (fp=fopen(sIniFileName,"r"))==NULL) 
	{
			return -1;
	}

	while (!feof(fp)) 
	{
		char *p;
		unsigned int n;

		if (fgets(line,sizeof(line),fp)==NULL) 
			break; ///< end-of-file 

		if ( strlen(line)<3)
			continue; 

		n=0;
		while (line[n]<=' ' && n<strlen(line))
			n++;

		if (line[n]=='#') 
			// comment 
			continue;

		if ( ( p=strchr( line , '=' )) == NULL) 
		{
			cout << "Warning: line ignored: " << line << endl;
			continue;
		}

		*p++=0;
		if ( strlen(line)>OPT_MAXLEN)		//32
		{
			cout << "Warning: unknown option: " << line <<endl;
			continue;
		}

		if ( strlen(p)>ARG_MAXLEN)		  //1024
		{
			cout << "Warning: option value too long:" << p <<endl;
			continue;
		}

		strcpy(opt,line);
		strcpy(arg,p);

		if ( strlen(arg)<2) 
			continue;

		if ( strchr(arg,' '))
			*strchr(arg,' ')=0;

		if ( strchr(arg,'\r')) 
			*strchr(arg,'\r')=0;


		switch (Conf::Optindex(opt)) 
		{
			case -1:
				cout << "Warning unknown option: "<<opt << endl;
				break;
			case OPT_COMMENT:
				break;
			
			case OPT_PORT:												//0
				Conf::sServerPort=atoi(arg);
				break;
			case OPT_MAXUSERS:											//1
				Conf::sMaxUsers=atoi(arg);
				if (Conf::sMaxUsers>100) 
				{
					cout <<"MAXUSERS limit is currently: " << HARD_USERLIMIT <<endl;
					Conf::sMaxUsers=HARD_USERLIMIT;
				}
				break;

			case OPT_ROOT:												//2
				if (strlen(Conf::s_ServerRoot)>MAXPATHLEN) 
				{
					cout << "s_serverroot option argument is too long."<<endl;
					fclose(fp);
					return -1;
				}
				strcpy(Conf::s_ServerRoot,arg);
				if(Conf::s_ServerRoot[strlen(Conf::s_ServerRoot)-1]=='/')
					Conf::s_ServerRoot[strlen(Conf::s_ServerRoot)-1]=0;
				break;
			case OPT_LOGINTIMEOUT:										//3
				Conf::sLoginTimeOut=atoi(arg);
				break;

/*			case OPT_USER:												//4
				sadd_user(arg);
				break;                                               
*/
			case OPT_TIMEOUT:											//5
				Conf::sIdleTimeOut=atoi(arg);
				break;
			case OPT_MODE:												//6
	
//				cout << OPT_MODE << endl;
				if (arg==NULL) 
					break;
				if (!strcasecmp(arg,"DAEMON")) 
				{
					sDaemonMode=0; //daemon 모드 적용
				}
				break;

			/*	case OPT_LOG:												//7
				if (arg==NULL || strlen(arg)==0) 
					break;
				slogfile=fopen(arg,"a");
				break;*/	
			case OPT_MOTD:												//8
				if (arg==NULL || strlen(arg)==0)
				{
					Conf::sMotdFile[0]=0;
					break;
				}
				if ( strlen(arg)>MAXPATHLEN) 
				{
					cout << "S_motd file argument is too long." << endl;
					fclose(fp);
					return -1;
				}
				strcpy(Conf::sMotdFile,arg);
				break;

			case OPT_BYEMSG:											//9
				if (arg==NULL || strlen(arg)==0) 
				{
					Conf::sByeMsgFile[0]=0;
					break;
				}
				if ( strlen(arg)>MAXPATHLEN) 
				{
//					cout <<"S_bymsg file argument is too long."<<endl;
					fclose(fp);
					return -1;
				}
				strcpy(sByeMsgFile,arg);
				break;
/*			case OPT_BAN:												//10
				sadd_ban(arg);
				break;
*/	
			case OPT_BANMSG:											//11
				if (arg==NULL || strlen(arg)==0) 
				{
					Conf::sBanMessage[0]=0;
					break;
				}
				if ( strlen(arg)<=70)
					strcpy(Conf::sBanMessage,arg);
				else 
				{
					cout << "Warning: S_Ban message is longer than 70 characters." << endl;
					fclose(fp);
					return -1;
				}
				break;
			case OPT_CHANGEROOTTYPE:									//12
				Conf::sRealChangeRoot=1; // default is REAL, default in Config is VIRTUAL 
				if (arg==NULL || strlen(arg)==0) 
					break;

				if (!strcasecmp(arg,"VIRTUAL")) 
					Conf::sRealChangeRoot=0;
				break;
		}
	}
	fclose(fp);
	return 0;
}

