/**
		@file 	servermanager.cpp	
		@date	2009.08.05
		@author	최영업(cyu0070@naver.com) Terapi
		@brief

*/
using namespace std;

#include "servermanager.h"
#define DEBUG_SERVERMANAGER
const char *SERVER_PROMPT="*ftpd> ";

const char *commands[CMD_COUNT]={
	"HELP","QUIT","LIST","WHO","KILL","STOP","ROOT","ADD",
	"DEL","REG","PERM","INIT","DOWN","FDOWN","BAN","UNBAN"
};

const char *commandlonghelp[CMD_COUNT]={
	"Syntax: help [command]\n\n'help' alone will display all available \
	commands.\n'help command' will give you some info about a command.\n\n\
	Example: help list\n",
	"Syntax: quit\n\n'quit' will shut down the server.\n",
	"Syntax: list\n\n'list' will display open connections and their status.\n",
	"Syntax: who [nr]\n\n'who' alone will give you information about users.\n\
	'who <nr>' will give you information about user with the specified number.\n",
	"Syntax: kill <nr>\n\n'kill' closes ALL connections to a user.\n",
	"Syntax: stop <nr>\n\n'stop' stops a data transfer to/from a user.\n\
	it doesnt close his login shell.\n",
	"Syntax: root <nr> <path>\n\n'root' changes a users root directory \
	while he is logged in.\nIt doesn't affect the login database.\n\
	(It might be confusing for the user, be careful!)\n",
	"Syntax: add <login> <passwd> <subdir> <maxlogins> <flags>\n\n\
	'add' will add a new login to the current login database.\n\n\
	Example: add jimmy0 hyper /tmp 5 DU\n\
	This command adds a new user with the login 'jimmy0' which has to \n\
	identify with password 'hyper'. His root directory will be '/tmp'.\n\
	He can use up to 5 logins at once. He will be allowed to download \n\
	and upload files.\n\n\
	Example: add john12 * /home/john12 0 M\n\
	This command adds a new user with the login 'john12' which doesn't need to \n\
	identify with a password. His root directory will be '/home/john12'.\n\
	He can use unlimited logins at once and has got ALL permissions.\n",
	"Syntax: del <login>\n\n'del' will delete the user <login> from the \
	current login database.\nIt doesn't modify the rc-file.\n",
	"Syntax: reg\n\n'reg' will display all available login from the currently \
	used login database.\n",
	"Syntax: perm <nr> <permissions>\n\n'perm' will modify the permissions of \
	a login account in the current login database.\nIt doesn't modify the \
	rc-file.\n\nPermission types:\n\
	D = download\n\
	U = upload\n\
	O = overwrite\n\
	M = multiple logins\n\
	E = erase\n\
	A = MASTER MODE (Be careful! It enables EVERYTHING!)\n",
	"Syntax: init\n\n'init' closes all connections, rereads the rc-file \
	and restarts \nthe server. (Be careful! It has the same effect as \
	restarting the process.)\n",
	"Syntax: down\n\n'down' will set the server to restricted mode.\n\
	That means that no further logins from non-MASTER users will be accepted.\n",
	"Syntax: fdown\n\n'fdown' will set the server to down mode.\n\
	That means that no further logins from ANY even MASTER users will be \
	accepted.\n",
	"Syntax: ban [mask]\n\n'ban' will show the current server ban-list.\n\
	'ban <mask>' will ban servers by name or IP. \"*\" and \"?\" are allowed.\n",
	"Syntax: unban [<nr>]\n\n'unban' will show the current server ban-list.\n\
	'unban <nr>' will unban the user number <nr> from the ban-list.\n"
};
const char *commandhelp[16]={
	"help",
	"exit server",
	"list connections",
	"info about user",
	"kill user's connection",
	"stop user's down-/upload",
	"change a user's root directory",
	"add a new user",
	"deletes a user",
	"list registered users",
	"sets a user's permissions",
	"restart server",
	"sets restricted mode",
	"forces down mode (no more operations)",
	"bans users from server",
	"removes bans from server"
};

const char *spacer="        ";
void ServerManager::docmd_help(char *arg) ///<server에서 Help명령어를 처리하는	함수
{
	printf("command-----------docmd_help-\n");
	int n;

	if (arg==NULL) 
	{
		printf("\nAvailable commands:\n");
		for (n=1;n<CMD_COUNT;n++)
			printf("%s%s - %s\n",commands[n],&spacer[strlen(commands[n])], commandhelp[n]);

		printf("\n");
	}
	else 
	{
		for (n=0;n<strlen(arg);n++)
			arg[n]=toupper(arg[n]);

		if (strlen(arg)<1) 
		{
			printf("Use: help [command]\n");
			return;
		}
		for (n=0;n<strlen(arg);n++)
			if (arg[n]==' ') 
				break;

		if (arg[n]==' ')
			arg[n]=0;


		for (n=0;n<CMD_COUNT;n++)
			if (!strcmp(commands[n],arg)) 
				break;

		if (n<CMD_COUNT)
			printf("\n%s\n",commandlonghelp[n]);
		else
			printf("\nSorry, I don't know anything about this command.\n\n");
	}
}
void ServerManager::docmd_list(UserManager **uarr) ///<현재 접속중인 유져의	idletime 을 확인하는 함수
{
	Accepter ac;
	int i,nr=0;
	int n=0;


	for (n=0;n<ac.s_Cur_User_cnt;n++) 
	{
		if (uarr[n]->sock) 
		{
//*//			printf("User %d ",n+1);
			nr++;
			switch (uarr[n]->state) 
			{
			case USR_USER:
				break;
			case USR_PASS:
				break;
			default:
				if (uarr[n]->pid==0 || waitpid(uarr[n]->pid,NULL,WNOHANG)<0) 
				{
					i=time(NULL)-uarr[n]->idletime;
	//*//					printf("idle: %d secs.\n",(int)i);
				}
				else 
		//*//				printf("uses pid: %d.\n",uarr[n]->pid);
				break;
			}
		}
	}
	if (nr==0) 
		printf("No users connected.\n");

//	return 0;
}
void ServerManager::docmd_who(UserManager **uarr,char *arg) ///<접속한 유저를 전체 확인할때 사용하는 함수
{
	//*//	printf("command----------.c-docmd_who-\n");
	int n,nr=0;

	if (arg==NULL || strlen(arg)==0) 
	{
		nr=0;
		for (n=0;n<Accepter::s_Cur_User_cnt;n++) 
		{
			if (uarr[n]->sock) 
			{
	//*//				printf("User %d, %s (%d.%d.%d.%d)\n",n+1,uarr[n]->url,uarr[n]->userip[0],uarr[n]->userip[1],uarr[n]->userip[2],uarr[n]->userip[3]);
				nr++;
			}
		}
		if (nr==0) 
			printf("No users connected.\n");
	}
	else 
	{
		n=atoi(arg)-1;
		if (n>=0 && n<Accepter::s_Cur_User_cnt && uarr[n]->sock) 
		{
			printf("User %d, %s (%d.%d.%d.%d)\n",n+1,uarr[n]->url,
				uarr[n]->userip[0],uarr[n]->userip[1],
				uarr[n]->userip[2],uarr[n]->userip[3]);
		} 
		else 
			printf("No such user.\n");
	}
}
void ServerManager::docmd_kill(UserManager **uarr,char* msg_buf,char *arg) ///<특정유저를	삭제할때
{
	printf("command----------.c-docmd_kill-\n");

	Handler hand;
	int nr;

	if (arg==NULL || strlen(arg)==0) 
	{
		printf("Kill <usernr>\n");
		return;
	}
	else 
	{
		nr=atoi(arg);
		if (nr==0 || nr>=Accepter::s_Cur_User_cnt) return;
		hand.kill_connection(uarr,msg_buf,nr-1);
	}
}
void ServerManager::docmd_stop(UserManager **uarr,pthread_mutex_t mutex,char* msg_buf,char *arg)
{
	printf("command----------.c-docmd_stop-\n");
	Handler hand;
	int nr;

	if (arg==NULL || strlen(arg)==0) 
	{
		printf("Stop <usernr>\n");
		return;
	}
	else 
	{
		nr=atoi(arg)-1;
		if (nr<0) 
			return;
		/* checking retr subprocess */
		if (uarr[nr]->pid) 
			kill(uarr[nr]->pid,SIGTERM);
		uarr[nr]->pid=0;
		if (uarr[nr]->pasvsock) 
		{
			close(uarr[nr]->pasvsock);
			uarr[nr]->pasvsock=0;
		}
		hand.user_return(uarr,mutex,msg_buf,nr,RET_591);
	}
}
void ServerManager::docmd_quit(UserManager **uarr,pthread_mutex_t mutex,char* msg_buf) ///< 서버 종료함수
{
	Handler hand;
	printf("command----------.c-docmd_quit-\n");
	int nr;

	for (nr=0;nr<Accepter::s_Cur_User_cnt;nr++)
	{
		if (uarr[nr]->sock) 
		{
			hand.user_return(uarr,mutex,msg_buf,nr,RET_428);
			hand.kill_connection(uarr,msg_buf,nr);
		}
	}
	FtpServer::s_Server_Stop=1;
//	serverstop=1;
}
void ServerManager::docmd_init(UserManager **uarr,pthread_mutex_t mutex,char* msg_buf) ///< 서버 동작중 초기화 함수
{
	printf("command----------.c-docmd_init-\n");
	int nr;
	Handler hand;

	for (nr=0;nr<Accepter::s_Cur_User_cnt;nr++)
	{
		if (uarr[nr]->sock) 
		{
			hand.user_return(uarr,mutex,msg_buf,nr,RET_429);
			hand.kill_connection(uarr,msg_buf,nr);
		}
	}
FtpServer::s_Server_Stop=1;
///	serverstop=1;
	//reinit=1;
}
void ServerManager::docmd_reg(void)
{
	printf("command----------.c-docmd_reg-\n");
	int n,m;
	char perm[16];

	for (n=0;n<Conf::sLoginInfoCount;n++)
	{
/*		printf("%d: %s %s (max: ",n+1,logininfo[n]->username,
			logininfo[n]->root);

		if (logininfo[n]->maxlogins) 
			printf("%d",logininfo[n]->maxlogins);
		else 
			printf("infinite");

		m=logininfo[n]->perm;
*/		if (m==PERM_ALL)
			strcpy(perm,"A");
		else 
		{
			perm[0]=0;
			
			if (m&PERM_DOWNLOAD) 
				strcat(perm,"D");
			if (m&PERM_UPLOAD) 
				strcat(perm,"U");
			if (m&PERM_ERASE) 
				strcat(perm,"E");
			if (m&PERM_OVERWRITE) 
				strcat(perm,"O");
			if (m&PERM_MULTIPLE) 
				strcat(perm,"M");
			if (!perm[0]) 
				strcat(perm,"-");
		}
		printf(") [%s]\n",perm);
	}
}





void ServerManager::out_prompt(void)
{
	cout << SERVER_PROMPT  ;
	fflush(stdout);
}

void ServerManager::Server_cmd(UserManager **uarr,pthread_mutex_t mutex,char* msg_buf,char *cmd)	///< 서버에서들어온 명령어를 판단하는 함수
{


#ifdef DEBUG_FTPSERVER1
	cout << "execcmd"<< endl;
#endif

	Handler hand;


	char *param=NULL;
	int nr;

	if (strlen(cmd)<2) 
		return;
	
	for (nr=0; nr<strlen(cmd); nr++)
		if (cmd[nr]==' ') 
			break;

	if (cmd[nr]==' ')
	{
		cmd[nr]=0;
		param=cmd+nr+1;
	}

	for (nr=0;nr<strlen(cmd);nr++)
		cmd[nr]=toupper(cmd[nr]);

	for (nr=0;nr<16;nr++)
		if (!strcmp(commands[nr],cmd)) 
			break;
	cout <<"command "<<cmd << " " <<"nr "<<nr<<endl;

	switch (nr) 
	{
		case CMD_LIST:
			docmd_list(uarr);
			break;
		case CMD_HELP:
			docmd_help(param);
			break;
		case CMD_KILL:
			docmd_kill(uarr,msg_buf,param);
			break;
		case CMD_STOP:
			docmd_stop(uarr,mutex,msg_buf,param);
			break;
		case CMD_QUIT:
			docmd_quit(uarr,mutex,msg_buf);
			break;
		case CMD_WHO:
			docmd_who(uarr,param);
			break;
		case CMD_ROOT:
//			docmd_root(uarr,msg_buf,param);
			break;
		case CMD_ADD:
//			add_user(param);
			break;
		case CMD_DEL:
//			docmd_del(param);
			break;
		case CMD_REG:
			docmd_reg();
			break;
		case CMD_PERM:
//			docmd_perm(param);
			break;
		case CMD_INIT:
			docmd_init(uarr,mutex,msg_buf);
			break;
		case CMD_DOWN:
//			docmd_down(uarr,1);
			break;
		case CMD_FDOWN:
//			docmd_down(uarr,0);
			break;
		case CMD_BAN:
//			add_ban(param);
			break;
		case CMD_UNBAN:
//			remove_ban(param);
			break;
		default:
			cout<<"I don't know about the command "<< cmd << endl;
			cout<<"Use HELP to display available commands"<<endl;
			break;
	}
}


