/**
		@file 	usermanager.cpp
		@data	2009.08.07
		@author	최영업(cyu0070@naver.com) Terapi
		@brief

*/
using namespace std;
#include "usermanager.h"



#define DEBUG_USERMANAGER
#define OPT_MAXLEN 32
#define ARG_MAXLEN 1024



char LoginInfo::s_username[5]="NULL"; ///< UserID
char LoginInfo::s_passwd[5]="NULL"; ///< UserPassWord
char LoginInfo::s_root[10]="NULL"; ///< 로긴시에 접속가능한 디렉토리
int  LoginInfo::s_perm=0; ///< permisstion에 대한 정보
int  LoginInfo::s_maxlogins=0; ///< 접속가능한 인원수


/// 각 유저마다 permission을 세팅한다.
int setpermissions(char *); 
///퍼미션에 따라서 해당값을 변환 해주는 함수
int translatepermissions(char *);


/**
	@return 변환된 permission value
	@warning
  */

int translatepermissions(char *perm)
{	 
	int n,p=PERM_NONE;
	if (perm==NULL) 
            return PERM_NONE;

	for (n=0;n<strlen(perm);n++)
		switch (perm[n]) 
	{
		case 'D':
			p|=PERM_DOWNLOAD;
			break;
		case 'U':
			p|=PERM_UPLOAD;
			break;
		case 'O':
			p|=PERM_OVERWRITE;
			break;
		case 'M':
			p|=PERM_MULTIPLE;
			break;
		case 'E':
			p|=PERM_ERASE;
			break;
		case 'A':
			p|=PERM_ALL;
			break;
		default:
			return -1;
	}
	return p;
}
/**
	@return 0
	@warning
  */

int setpermissions(char *perm)
{
	LoginInfo login;
	int p;

	p=translatepermissions(perm);
	login.s_perm= p<0 ? PERM_NONE : p;
	return 0;
}


/**
	@return 사용자개정 등록 성공 0 실패 -1
	@warning 예외처리 안되어있음
			 userinfo.ini 가 잘못되어있을경우 예외처리 현재없음
  */
int LoginInfo::m_Add_User(char *msg_buf,char *arg)
{

	LoginInfo login;
	int logininfocount=0;
	char t_perm[5];

	if (logininfocount<LOGININFO_COUNT) 
	{
		if (arg==NULL ||sscanf(arg,"%s %s %s %d	%s",&login.s_username, &login.s_passwd, &login.s_root, &login.s_maxlogins, &t_perm ) < 5 ) 
		{
			cout<<"Invalid user registration format."<<endl;
			cout<<"The correct format is: <login> <passwd> <subdir> <maxlogins>	<flags>"<<endl;
			return -1; //조건식 불가경우
		}
		setpermissions(t_perm);
		if(Conf::sDaemonMode)
			cout<<"User "<<login.s_username<<" registered."<<endl;;

		logininfocount++;
		return 0;
	} 
	else
	{
		cout<<"Warning: Too many users (> "<<LOGININFO_COUNT <<" )specified."<<endl;
		return -1;	//등록인원수가 많을경우
	}
}


/***
  @return		userinfo[]배열안에 들어있는 문자열과 비교하여서 맞으면 해당하는 배열값
 **/
int LoginInfo::m_Optindex(char *opt) 
{
	
	int n;

	n=0;
	while (opt[n]==' ' && n<strlen(opt)) 
		n++;

	if (opt[n]=='#' || opt[n]==0)      
		return OPT_COMMENT;

	if (!strcasecmp(opt,"USER"))
		return 0;
	return -1;
}
/**
	@return 성공이면 0, 실패면 -1
  */

int LoginInfo::m_ReadUserInfo(char* msg_buf )		
{
	FILE *fp;
	char line[2048],opt[OPT_MAXLEN+4],arg[ARG_MAXLEN+4];
	static char userinifilename[512];	

	LoginInfo login;

	strcpy(userinifilename,DEFAULT_USERINFOPATH);

	if ((fp=fopen(userinifilename,"r"))==NULL) 
	{
		cout<<"Cannot find "<<userinifilename<<endl;
		cout<<"Make sure it's available."<<endl;
		return -1;
	}

	while (!feof(fp)) 
	{
		char *p;
		unsigned int n=0;

		if (fgets(line,sizeof(line),fp)==NULL) 
			break; // end-of-file 

		if (strlen(line)<3)
			continue; 

		// line empty 
		// check if comment 	  
		while (line[n]<=' ' && n<strlen(line))
			n++;

		if (line[n]=='#') 									//주석처리
			continue;

		if ((p=strchr(line , '=' )) == NULL) 
		{
			cout<<"Warning: line ignored: "<<line<<endl;
			continue;
		}
		*p++=0;
		if (strlen(line)>OPT_MAXLEN)						//32
		{
			cout<<"Warning: unknown option: "<<line<<endl;
			continue;
		}

		if (strlen(p)>ARG_MAXLEN)		  					//1024
		{
			cout<<"Warning: option value too long: "<<p<<endl;
			continue;
		}

		strcpy(opt,line);
		strcpy(arg,p);

		if (strlen(arg)<2) 
			continue;

		if (strchr(arg,'\n'))
			*strchr(arg,'\n')=0;

		if (strchr(arg,'\r')) 
			*strchr(arg,'\r')=0;

		switch (login.m_Optindex(opt)) 
		{
			case -1:
				cout<<"Warning unknown option: "<<opt<<endl;
				break;
			case OPT_COMMENT:
				break;
			case 0:
				login.m_Add_User(msg_buf,arg);
				break;
		}
	}
	fclose(fp);
	return 0;
}



UserManager::UserManager()			///<UserManager 에 대한 디폴트 생성자
{
	sock     = 0;
	idletime = 0;
	usertype = 0;
	pid      = 0;
	resume   = 0;
	state    = 0;
	perm     = 0;
	dataport = 0;
	pasvsock = 0;
	*userip  = 0;
	*dataip  = 0;
	*rootdir ='\0';
	*reldir  ='\0';
	*url	='\0';
};

UserManager::UserManager(
		int _sock,  int _idletime, int _usertype,               int _pid,      int _resume,		 
		int _state, int _perm,     unsigned short int _dataport,int _pasvsock,
		int*  _userip,	int*  _dataip,	char* _rootdir, char* _reldir, 		char* _url
		)
{
	sock     = _sock;
	idletime = _idletime;
	usertype = _usertype; 
	pid      = _pid;
	resume   = _resume;		 
	state    = _state;
	perm     = _perm; 
	dataport = _dataport;
	pasvsock = _pasvsock;

	userip = new int[4];	
	dataip = new int[4];
	rootdir = new char[1024];	strcpy(rootdir,_rootdir);
	reldir  = new char[1024]; 	strcpy(reldir ,_reldir);
	url  	= new char[1024]; 	strcpy(url ,_url);

}; 

UserManager::~UserManager()
{
	delete []userip;
	delete []dataip;
	delete []rootdir;
	delete []reldir;
	delete []url;
}



